@@ -1,9 +1,7 @@
 <?php
 /*
- * $RCSfile: index.php,v $
- *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2006 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,18 +18,17 @@
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
  */
 /**
- * @version $Revision: 1.19 $ $Date: 2006/03/06 05:33:32 $
+ * This script will just redirect to main.php
+ * The Location header requires an absolute url to conform with HTTP/1.1
  * @package Gallery
  * @author Bharat Mediratta <bharat@menalto.com>
+ * @version $Revision: 15513 $
  */
 
-/*
- * This script will just redirect to main.php. But the Location header requires
- * an absolute url to conform with HTTP/1.1
- * (Include bootstrap.inc in case config.php overrides GALLERY_MAIN_PHP)
- */
+/* Include bootstrap.inc in case config.php overrides GALLERY_MAIN_PHP */
 require_once(dirname(__FILE__) . '/bootstrap.inc');
 require_once(dirname(__FILE__) . '/modules/core/classes/GalleryUrlGenerator.class');
+require_once(dirname(__FILE__) . '/modules/core/classes/GalleryCoreApi.class');
 
 /* The REQUEST_URI can either be /path/index.php or just /path/. Get rid of index.php.* */
 $path = GalleryUrlGenerator::getCurrentRequestUri();
@@ -44,8 +41,9 @@ if (preg_match('|^(/(?:[^?#/]+/)*)(.*)|', $path, $matches)) {
 
 $configBaseUri = @$gallery->getConfig('baseUri');
 
-$urlGenerator =& new GalleryUrlGenerator();
+$urlGenerator = new GalleryUrlGenerator();
 $urlGenerator->init(!empty($configBaseUri) ? $configBaseUri : null);
 
-header('Location: ' . $urlGenerator->makeUrl($path));
+$phpVm = $gallery->getPhpVm();
+$phpVm->header('Location: ' . $urlGenerator->makeUrl($path));
 ?>
