@@ -10,7 +10,7 @@
  *
  * ----------------------------------------------------------------------------
  *
- * $Id: index.php 15711 2007-01-19 18:50:16Z bharat $
+ * $Id: index.php 17265 2007-12-22 21:58:40Z andy_st $
  *
  * Gallery - a web based photo album viewer and editor
  * Copyright (C) 2000-2007 Bharat Mediratta
@@ -277,10 +277,8 @@ function secureStorageFolder($dataBase) {
 			    "<IfModule mod_rewrite.c>\n" .
 			    "RewriteEngine off\n" .
 			    "</IfModule>\n" .
-			    "<IfModule mod_access.c>\n" .
 			    "Order allow,deny\n" .
-			    "Deny from all\n" .
-			    "</IfModule>\n";
+			    "Deny from all\n";
 	fwrite($fh, $htaccessContents);
 	fclose($fh);
     }
