@@ -1,8 +1,6 @@
 /*
- * $RCSfile: BlockToggle.js,v $
- *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2005 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,9 +25,9 @@ function BlockToggle(objId, togId, display) {
 	    if (agent.indexOf('msie') >= 0 && agent.indexOf('opera') < 0) display = 'block';
 	}
 	o.style.display = display;
-	t.innerHTML = '-';
+	if (t) t.innerHTML = '-';
     } else {
 	o.style.display = 'none';
-	t.innerHTML = '+';
+	if (t) t.innerHTML = '+';
     }
 }
