@@ -1,8 +1,6 @@
 #!/usr/bin/php -f
 <?php
 /*
- * $RCSfile: extract.php,v $
- *
  * PHP script to extract strings from all the files and print
  * to stdout for use with xgettext.
  *
@@ -23,9 +21,9 @@
  * way that Gallery embeds internationalized text, so let's tack on our
  * own copyrights.
  *
- * Copyright 2002-2006 Bharat Mediratta <bharat@menalto.com>
+ * Copyright 2002-2007 Bharat Mediratta <bharat@menalto.com>
  *
- * $Id: extract.php,v 1.21 2006/02/27 21:44:21 mindless Exp $
+ * $Id: extract.php 15513 2006-12-20 00:24:57Z mindless $
  */
 
 if (!empty($_SERVER['SERVER_NAME'])) {
@@ -39,7 +37,12 @@ $skip = array('TEST to be displayed in different languages' => true,
 	      'TT <!-- abbreviation for Translation Test -->' => true);
 $idEmitted = false;
 $strings = array();
+array_shift($_SERVER['argv']);
 foreach ($_SERVER['argv'] as $moduleDir) {
+    if (preg_match('#^/cygdrive/(\w+)/(.*)$#', trim($moduleDir), $matches)) {
+	/* Cygwin and Window PHP filesystem function don't play nice together. */
+	$moduleDir = $matches[1] . ':\\' . str_replace('/', '\\', $matches[2]);
+    }
     if (!is_dir($moduleDir)) {
 	continue;
     }
@@ -86,7 +89,10 @@ function find($dir) {
 	foreach ($listing as $file) {
 	    $filename = $dir . $file;
 	    if (is_dir($filename)) {
-		$subdirs[] = $filename;
+		/* Don't parse unit tests */
+		if ($file != 'test') {
+		    $subdirs[] = $filename;
+		}
 	    } else if (preg_match("/\." . $exts . "$/", $file)) {
 		extractStrings($filename);
 	    }
@@ -105,30 +111,26 @@ function extractStrings($filename) {
     $strings["\n/* $filename */"] = array();
     $startSize = count($strings);
     $localStrings = array();
-    if (function_exists('file_get_contents')) {
-	$data = file_get_contents($filename);
-    } else {
-	$fd = fopen($filename, 'r');
-	$fileSize = filesize($filename);
-	$data = $fileSize == 0 ? '' : fread($fd, $fileSize);
-	fclose($fd);
-    }
+    $data = file_get_contents($filename);
 
     /*
      * grab phrases for translate( or i18n( or _( calls; capture string parameter enclosed
      * in single or double quotes including concatenated strings like 'one' . "two"
      */
-    if (preg_match_all("/(translate|i18n|_)\(\s*(((\s*\.\s*)?('((\\')?[^']*?)*[^\\\]'|\"((\")?[^\"]*?)*[^\\\]\"))+)\s*\)/s",
+    if (preg_match_all("/(translate|i18n|_)\(\s*((?:(?:\s*\.\s*)?(?:'(?:(?:\\')?[^']*?)*[^\\\\]'|\"(?:(?:\")?[^\"]*?)*[^\\\\]\"))+)\s*(?:,\s*(true|false)\s*)?\)/s",
 		       $data, $matches, PREG_SET_ORDER)) {
 	foreach ($matches as $match) {
-	    $text = $match[2];
-	    $cmd = sprintf('return %s;', $text);
-	    $text = eval($cmd);
+	    $text = eval('return ' . $match[2] . ';');
 	    $text = str_replace('"', '\\"', $text);    /* escape double-quotes */
 	    if (isset($skip[$text])) {
 		continue;
 	    }
-	    $string = sprintf('gettext("%s")', $text);
+	    if (!empty($match[3])) {
+		$hint = '/* xgettext:' . ($match[3] == 'false' ? 'no-' : '') . "c-format */\n";
+	    } else {
+		$hint = '';
+	    }
+	    $string = $hint . sprintf('gettext("%s")', $text);
 	    if (!isset($strings[$string])) {
 		$strings[$string] = array();
 	    } else if (!isset($localStrings[$string])) {
@@ -139,11 +141,11 @@ function extractStrings($filename) {
     }
 
     /* grab phrases of this format: translate(array('one' => '...', 'many' => '...')) */
-    if (preg_match_all("/translate\(.*?array\(\s*'one'\s*=>\s*'(.*?)'.*?'many'\s*=>\s*'(.*?)'.*?\).*?\)/s",
+    if (preg_match_all("/translate\(\s*array\(\s*'one'\s*=>\s*((?:(?:\s*\.\s*)?(?:'(?:(?:\\')?[^']*?)*[^\\\\]'|\"(?:(?:\")?[^\"]*?)*[^\\\\]\"))+).*?'many'\s*=>\s*((?:(?:\s*\.\s*)?(?:'(?:(?:\\')?[^']*?)*[^\\\\]'|\"(?:(?:\")?[^\"]*?)*[^\\\\]\"))+)\s*[,)]/s",
 		       $data, $matches, PREG_SET_ORDER)) {
 	foreach ($matches as $match) {
-	    $one = $match[1];
-	    $many = $match[2];
+	    $one = eval('return ' . $match[1] . ';');
+	    $many = eval('return ' . $match[2] . ';');
 	    $one = str_replace('"', '\\"', $one);      /* escape double-quotes */
 	    $many = str_replace('"', '\\"', $many);    /* escape double-quotes */
 	    $string = sprintf('ngettext("%s", "%s")', $one, $many);
@@ -157,10 +159,10 @@ function extractStrings($filename) {
     }
 
     /* grab phrases of this format: translate(array('text' => '...', ...)) */
-    if (preg_match_all("/translate\(\s*array\(\s*'text'\s*=>\s+'(.*?[^\\\])'/s",
+    if (preg_match_all("/translate\(\s*array\(\s*'text'\s*=>\s*((?:(?:\s*\.\s*)?(?:'(?:(?:\\')?[^']*?)*[^\\\\]'|\"(?:(?:\")?[^\"]*?)*[^\\\\]\"))+)\s*[,)]/s",
 		       $data, $matches, PREG_SET_ORDER)) {
 	foreach ($matches as $match) {
-	    $text = $match[1];
+	    $text = eval('return ' . $match[1] . ';');
 	    $text = str_replace('"', '\\"', $text);    /* escape double-quotes */
 	    $string = sprintf('gettext("%s")', $text);
 	    if (!isset($strings[$string])) {
