@@ -1,7 +1,7 @@
 /****************************
  * Hybrid theme for Gallery2
  * @author Alan Harder <alan.harder@sun.com>
- * $Revision: 1.10 $ $Date: 2005/10/14 18:29:18 $
+ * $Revision: 15780 $
  */
 
 //Class app
@@ -9,6 +9,7 @@ var app_ww, app_wh, // Window width/height
     app_agent = navigator.userAgent.toLowerCase(), // Browser type
     app_version = parseInt(navigator.appVersion),
     app_is_ie = app_agent.indexOf('msie') >= 0 && app_agent.indexOf('opera') < 0,
+    app_is_ie7 = app_agent.indexOf('msie 7') >= 0 && app_agent.indexOf('opera') < 0,
     app_is_safari = app_agent.indexOf('safari') >= 0,
     app_body; // Scrollable document container (<body> element, or html for IE)
 if (window.attachEvent) {
@@ -19,7 +20,7 @@ if (window.attachEvent) {
   window.addEventListener("unload", app_setcookie, false);
 }
 function app_init() {
-  if (!app_is_ie && !app_is_safari) album_setfixedtitle();
+  if (!app_is_ie && !app_is_safari || app_is_ie7) album_setfixedtitle();
   app_body = document.getElementById('hybridMain');
   while (app_body && app_body.tagName != 'BODY') app_body = app_body.parentNode;
   if (app_is_ie) app_body = app_body.parentNode;
@@ -223,6 +224,9 @@ function sidebar_onoff() {
   ui_vis('sidebar_max', sidebar_on, 1);
   ui_vis('sidebar_min', (sidebar_on = sidebar_on?0:1), 1);
   ui_vis('sidebar', sidebar_on);
+  if (app_is_safari) {
+    document.getElementById('sidebar').parentNode.style.width = sidebar_on ? 'auto' : '0';
+  }
 }
 
 //Class image :: div imageview(imagearea(imagediv),textdiv(tools_left,tools_right,title,text))
@@ -372,7 +376,7 @@ function slide_previndex() {
   return slide_nextindex(1);
 }
 function slide_next() {
-  var i = slide_nextindex(); if (i >= 0) image_show(i); else slide_onoff();
+  var i = slide_nextindex(); if (i >= 0) image_show(i); else { slide_inprog = 0; slide_onoff(); }
 }
 function slide_setorder(o) {
   slide_order = parseInt(o);
