@@ -10,10 +10,10 @@
  *
  * ----------------------------------------------------------------------------
  *
- * $Id: index.php,v 1.33 2006/03/10 00:15:34 mindless Exp $
+ * $Id: index.php 15513 2006-12-20 00:24:57Z mindless $
  *
  * Gallery - a web based photo album viewer and editor
- * Copyright (C) 2000-2006 Bharat Mediratta
+ * Copyright (C) 2000-2007 Bharat Mediratta
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
  */
+/**
+ * Gallery Upgrader
+ * @package Upgrade
+ */
 
 /* Show all errors. */
 @ini_set('display_errors', 1);
@@ -39,9 +43,13 @@
  */
 @ini_set('magic_quotes_runtime', 0);
 
-require_once(dirname(__FILE__) . '/UpgradeStep.class');
-require_once(dirname(__FILE__) . '/StatusTemplate.class');
-require_once(dirname(__FILE__) . '/../bootstrap.inc');
+$g2Base = dirname(dirname(__FILE__)) . '/';
+require_once($g2Base . 'upgrade/UpgradeStep.class');
+require_once($g2Base . 'upgrade/StatusTemplate.class');
+require_once($g2Base . 'bootstrap.inc');
+require_once($g2Base . 'modules/core/classes/GalleryUtilities.class');
+require_once($g2Base . 'lib/support/GallerySetupUtilities.class');
+
 
 /*
  * If gettext isn't enabled, subvert the _() text translation function
@@ -70,51 +78,7 @@ foreach ($stepOrder as $stepName) {
     require("steps/$className.class");
 }
 
-/* Prevent session fixation attack by only accepting sessionIds of existing sessions */
-$sessionName = session_name();
-$sessionId = GalleryUtilities::getRequestVariablesNoPrefix($sessionName);
-if (empty($sessionId)) {
-    $sessionId = !empty($_COOKIE[$sessionName]) ? $_COOKIE[$sessionName] : '';
-}
-/* Remember whether cookies are supported */
-areCookiesSupported();
-/* Sanitize the sessionId */
-if (!empty($sessionId)) {
-    if (function_exists('preg_replace')) {
-	$sessionId = preg_replace('/[^a-zA-Z0-9]/', '', $sessionId);
-    } else {
-	$sessionId = ereg_replace('/[^a-zA-Z0-9]/', '', $sessionId);
-    }
-    /* Make sure we don't use invalid data at a later point */
-    foreach (array($_GET, $_POST, $_REQUEST, $_COOKIE) as $superGlobal) {
-	unset($superGlobal[$sessionName]);
-    }
-    /*
-     * md5 has a 128 bit (32 * 4bit) string, but we want to allow for other possible
-     * hash functions too which possibly have hash strings of only 10 characters
-     */
-    if (strlen($sessionId) >= 10) {
-	session_id($sessionId);
-    }
-}
-
-if (@ini_get('session.save_handler') != 'files') {
-    @ini_set('session.save_handler','files');
-    session_start();
-} else if (!ini_get('session.auto_start')) {
-    session_start();
-} /* In case of session.auto_start we can't prevent showing errors for invalid sessionIds */
-
-if (!isset($_SESSION['path'])) {
-    /* Empty session -> either new or a session fixation attack. Better regenerate */
-    regenerateSession();
-    $_SESSION['path'] = __FILE__;
-} else if ($_SESSION['path'] != __FILE__) {
-    /* Security error!  This session is not valid for this copy of the upgrader. Start over. */
-    regenerateSession();
-    session_unset();
-    $_SESSION['path'] = __FILE__;
-}
+GallerySetupUtilities::startSession();
 
 require_once(dirname(__FILE__) . '/../init.inc');
 /* Check if config.php is ok */
@@ -233,27 +197,39 @@ if ($currentStep->processRequest()) {
 
 /**
  * Find admin user and set as active user
+ * @param bool $fallback (optional) whether we should try to fall back if the
+ *             API to load the admin user object fails
  * @return object GalleryStatus a status code
  */
-function selectAdminUser() {
+function selectAdminUser($fallback=false) {
     global $gallery;
 
     list ($ret, $siteAdminGroupId) =
 	GalleryCoreApi::getPluginParameter('module', 'core', 'id.adminGroup');
     if ($ret) {
-	return $ret->wrap(__FILE__, __LINE__);
+	return $ret;
     }
     list ($ret, $adminUserInfo) = GalleryCoreApi::fetchUsersForGroup($siteAdminGroupId, 1);
     if ($ret) {
-	return $ret->wrap(__FILE__, __LINE__);
+	return $ret;
     }
     if (empty($adminUserInfo)) {
-	return GalleryCoreApi::error(ERROR_MISSING_VALUE, __FILE__, __LINE__);
+	return GalleryCoreApi::error(ERROR_MISSING_VALUE);
     }
-    $adminUserInfo = array_keys($adminUserInfo);
-    list ($ret, $adminUser) = GalleryCoreApi::loadEntitiesById($adminUserInfo[0]);
+    /* Fetch the first admin from list */
+    list ($userId, $userName) = each($adminUserInfo);
+    list ($ret, $adminUser) = GalleryCoreApi::loadEntitiesById($userId);
     if ($ret) {
-	return $ret->wrap(__FILE__, __LINE__);
+	if ($fallback) {
+	    /* Initialize a GalleryUser with the id of a real admin */
+	    $gallery->debug('Unable to load admin user. Using in-memory user object as fallback');
+	    GalleryCoreApi::requireOnce('modules/core/classes/GalleryUser.class');
+	    $adminUser = new GalleryUser();
+	    $adminUser->setId((int)$userId);
+	    $adminUser->setUserName($userName);
+	} else {
+	    return $ret;
+	}
     }
 
     $gallery->setActiveUser($adminUser);
@@ -276,7 +252,7 @@ function generateUrl($uri, $print=true) {
 	}
     } else if (!strncmp($uri, 'index.php', 9)) {
 	/* If session.use_trans_sid is on then it will add the session id. */
-	if (!areCookiesSupported() && !ini_get('session.use_trans_sid')) {
+	if (!GallerySetupUtilities::areCookiesSupported() && !ini_get('session.use_trans_sid')) {
 	    /*
 	     * Don't use SID since it's a constant and we change (regenerate) the session id
 	     * in the request
@@ -293,49 +269,6 @@ function generateUrl($uri, $print=true) {
     return $uri;
 }
 
-/**
- * Regenerate the session id to prevent session fixation attacks
- * Must be called before starting to output any data since it tries to send a cookie
- */
-function regenerateSession() {
-    /* 1. Generate a new session id */
-    $newSessionId = md5(uniqid(substr(rand() . serialize($_REQUEST), 0, 114)));
-    $sessionData = array();
-    if (!empty($_SESSION) && is_array($_SESSION)) {
-	foreach ($_SESSION as $key => $value) {
-	    $sessionData[$key] = $value;
-	}
-    }
-    /* 2. Delete the old session */
-    session_unset();
-    session_destroy();
-    /* Create the new session with the old data, send cookie */
-    session_id($newSessionId);
-    $sessionName = session_name();
-    /* Make sure we don't use invalid data at a later point */
-    foreach (array($_GET, $_POST, $_REQUEST, $_COOKIE) as $superGlobal) {
-	unset($superGlobal[$sessionName]);
-    }
-    session_start();
-    foreach ($sessionData as $key => $value) {
-	$_SESSION[$key] = $value;
-    }
-}
-
-/**
- * Are cookies supported by the current user-agent?
- */
-function areCookiesSupported() {
-    static $areCookiesSupported;
-
-    /* Remember the state since we might unset $_COOKIE */
-    if (!isset($areCookiesSupported)) {
-	$areCookiesSupported = !empty($_COOKIE[session_name()]);
-    }
-
-    return $areCookiesSupported;
-}
-
 /*
  * We don't store the steps in the session in raw form because that
  * will break in environments where session.auto_start is on since
